class Users(object):
    def __init__(self, client):
        self._client = client

    def get_user(self, user_id, **kwargs):
        url = 'users/{}'.format(user_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_all_users(self, **kwargs):
        url = 'users'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_me(self, **kwargs):
        url = 'users/me'
        return self._client._get(self._client.BASE_URL + url, **kwargs)



class Pipelines(object):
    def __init__(self, client):
        self._client = client

    def get_pipeline(self, pipeline_id, **kwargs):
        url = 'pipelines/{}'.format(pipeline_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_all_pipelines(self, **kwargs):
        url = 'pipelines'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_pipeline_deals(self, pipeline_id, **kwargs):
        url = 'pipelines/{}/deals'.format(pipeline_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)



class Stages(object):
    def __init__(self, client):
        self._client = client

    def get_stage(self, stage_id, **kwargs):
        url = 'stages/{}'.format(stage_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_all_stages(self, **kwargs):
        url = 'stages'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_stage_deals(self, stage_id, **kwargs):
        url = 'stages/{}/deals'.format(stage_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def create_stage(self, data, **kwargs):
        url = 'stages'
        return self._client._post(self._client.BASE_URL + url, data, **kwargs)

    def update_stage(self, stage_id, data, **kwargs):
        url = 'stages/{}'.format(stage_id)
        return self._client._put(self._client.BASE_URL + url, data, **kwargs)

    def delete_stage(self, stage_id, **kwargs):
        url = 'stages/{}'.format(stage_id)
        return self._client._delete(self._client.BASE_URL + url, **kwargs)



class Persons(object):
    def __init__(self, client):
        self._client = client

    def get_person(self, person_id, **kwargs):
        url = 'persons/{}'.format(person_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_all_persons(self, **kwargs):
        url = 'persons'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_persons_by_name(self, **kwargs):
        url = 'persons/find'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def create_person(self, data, **kwargs):
        url = 'persons'
        return self._client._post(self._client.BASE_URL + url, json=data, **kwargs)

    def update_person(self, person_id, data, **kwargs):
        url = 'persons/{}'.format(person_id)
        return self._client._put(self._client.BASE_URL + url, json=data, **kwargs)

    def delete_person(self, person_id, **kwargs):
        url = 'persons/{}'.format(person_id)
        return self._client._delete(url, **kwargs)

    def get_person_deals(self, person_id, **kwargs):
        url = 'persons/{}/deals'.format(person_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_person_fields(self, **kwargs):
        url = 'personFields'
        return self._client._get(self._client.BASE_URL + url, **kwargs)



class Webhooks(object):
    def __init__(self, client):
        self._client = client

    def get_hooks_subscription(self, **kwargs):
        url = 'webhooks'
        return self._client._get(url, **kwargs)

    def create_hook_subscription(self, subscription_url, event_action, event_object, **kwargs):
        url = 'webhooks'
        data = {
            'subscription_url':
                subscription_url,
            'event_action': event_action,
            'event_object': event_object
        }
        return self._client._post(url, json=data, **kwargs)

    def delete_hook_subscription(self, hook_id, **kwargs):
        url = 'webhooks/{}'.format(hook_id)
        return self._client._delete(url, **kwargs)



class Products(object):
    def __init__(self, client):
        self._client = client

    def get_product(self, product_id, **kwargs):
        url = 'products/{}'.format(product_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_all_products(self, **kwargs):
        url = 'products'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_product_by_name(self, **kwargs):
        url = 'products/find'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def create_product(self, data, **kwargs):
        url = 'products'
        return self._client._post(self._client.BASE_URL + url, json=data, **kwargs)

    def update_product(self, product_id, data, **kwargs):
        url = 'products/{}'.format(product_id)
        return self._client._put(self._client.BASE_URL + url, json=data, **kwargs)

    def delete_product(self, product_id, **kwargs):
        url = 'products/{}'.format(product_id)
        return self._client._delete(self._client.BASE_URL + url, **kwargs)

    def get_product_deal(self, product_id, **kwargs):
        url = 'products/{}/deals'.format(product_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_product_fields(self, **kwargs):
        url = 'productFields'
        return self._client._get(self._client.BASE_URL + url, **kwargs)



class Notes(object):
    def __init__(self, client):
        self._client = client

    def get_note(self, note_id, **kwargs):
        url = 'notes/{}'.format(note_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_all_notes(self, **kwargs):
        url = 'notes'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def create_note(self, data, **kwargs):
        url = 'notes'
        return self._client._post(self._client.BASE_URL + url, json=data, **kwargs)

    def update_note(self, note_id, data, **kwargs):
        url = 'notes/{}'.format(note_id)
        return self._client._put(self._client.BASE_URL + url, json=data, **kwargs)

    def delete_note(self, note_id, **kwargs):
        url = 'notes/{}'.format(note_id)
        return self._client._delete(self._client.BASE_URL + url, **kwargs)

    def get_note_fields(self, **kwargs):
        url = 'noteFields'
        return self._client._get(self._client.BASE_URL + url, **kwargs)



class BaseError(Exception):
    def __init__(self, message, response, *args, **kwargs):
        super().__init__(message, *args, **kwargs)
        self.response = response


class BadRequestError(BaseError):
    pass


class UnauthorizedError(BaseError):
    pass


class ForbiddenError(BaseError):
    pass


class NotFoundError(BaseError):
    pass


class GoneError(BaseError):
    pass


class UnsupportedMediaTypeError(BaseError):
    pass


class UnprocessableEntityError(BaseError):
    pass


class TooManyRequestsError(BaseError):
    pass


class InternalServerError(BaseError):
    pass


class NotImplementedError(BaseError):
    pass


class ServiceUnavailableError(BaseError):
    pass


class UnknownError(BaseError):
    pass



class Deals(object):
    def __init__(self, client):
        self._client = client

    def get_deal(self, deal_id, **kwargs):
        url = 'deals/{}'.format(deal_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_all_deals(self, **kwargs):
        url = 'deals'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_all_deals_with_filter(self, filter_id, **kwargs):
        url = 'deals?filter_id={}'.format(filter_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def create_deal(self, data, **kwargs):
        url = 'deals'
        return self._client._post(self._client.BASE_URL + url, json=data, **kwargs)

    def update_deal(self, deal_id, data, **kwargs):
        url = 'deals/{}'.format(deal_id)
        return self._client._put(self._client.BASE_URL + url, json=data, **kwargs)

    def delete_deal(self, deal_id, **kwargs):
        url = 'deals/{}'.format(deal_id)
        return self._client._delete(self._client.BASE_URL + url, **kwargs)

    def duplicate_deal(self, deal_id, **kwargs):
        url = 'deals/{}/duplicate'.format(deal_id)
        return self._client._post(self._client.BASE_URL + url, **kwargs)

    def get_deal_details(self, deal_id, **kwargs):
        url = 'deals/{}'.format(deal_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_deals_by_name(self, **kwargs):
        url = 'deals/find'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_deal_followers(self, deal_id, **kwargs):
        url = 'deals/{}/followers'.format(deal_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def add_follower_to_deal(self, deal_id, user_id, **kwargs):
        url = 'deals/{}/followers'.format(deal_id)
        data = {
            'user_id': user_id
        }
        return self._client._post(self._client.BASE_URL + url, json=data, **kwargs)

    def delete_follower_to_deal(self, deal_id, follower_id, **kwargs):
        url = 'deals/{}/followers/{}'.format(deal_id, follower_id)
        return self._client._delete(self._client.BASE_URL + url, **kwargs)

    def get_deal_participants(self, deal_id, **kwargs):
        url = 'deals/{}/participants'.format(deal_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def add_participants_to_deal(self, deal_id, person_id, **kwargs):
        url = 'deals/{}/participants'.format(deal_id)
        data = {
            'person_id': person_id
        }
        return self._client._post(self._client.BASE_URL + url, json=data, **kwargs)

    def delete_participant_to_deal(self, deal_id, participant_id, **kwargs):
        url = 'deals/{}/participants/{}'.format(deal_id, participant_id)
        return self._client._delete(self._client.BASE_URL + url, **kwargs)

    def get_deal_activities(self, deal_id, **kwargs):
        url = 'deals/{}/activities'.format(deal_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_deal_mail_messages(self, deal_id, **kwargs):
        url = 'deals/{}/mailMessages'.format(deal_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_deal_products(self, deal_id, **kwargs):
        url = 'deals/{}/products'.format(deal_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_deal_fields(self, **kwargs):
        url = 'dealFields'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def add_product_to_deal(self, deal_id, data, **kwargs):
        url = 'deals/{}/products'.format(deal_id)
        return self._client._post(self._client.BASE_URL + url, json=data, **kwargs)



class Filters(object):
    def __init__(self, client):
        self._client = client

    def get_filter(self, filter_id, **kwargs):
        url = 'filters/{}'.format(filter_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_all_filters(self, **kwargs):
        url = 'filters'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def create_filter(self, data, **kwargs):
        url = 'filters'
        return self._client._post(self._client.BASE_URL + url, json=data, **kwargs)

    def update_filter(self, filter_id, data, **kwargs):
        url = 'filters/{}'.format(filter_id)
        return self._client._put(self._client.BASE_URL + url, json=data, **kwargs)

    def delete_filter(self, filter_id, **kwargs):
        url = 'filters/{}'.format(filter_id)
        return self._client._delete(self._client.BASE_URL + url, **kwargs)



class Activities(object):
    def __init__(self, client):
        self._client = client

    def get_activity(self, activity_id, **kwargs):
        url = 'activities/{}'.format(activity_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_all_activities(self, **kwargs):
        url = 'activities'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def create_activity(self, data, **kwargs):
        url = 'activities'
        return self._client._post(self._client.BASE_URL + url, json=data, **kwargs)

    def update_activity(self, activity_id, data, **kwargs):
        url = 'activities/{}'.format(activity_id)
        return self._client._put(self._client.BASE_URL + url, json=data, **kwargs)

    def delete_activity(self, activity_id, **kwargs):
        url = 'activities/{}'.format(activity_id)
        return self._client._delete(self._client.BASE_URL + url, **kwargs)

    def get_activity_fields(self, **kwargs):
        url = 'activityFields'
        return self._client._get(self._client.BASE_URL + url, **kwargs)



class Recents(object):
    def __init__(self, client):
        self._client = client

    def get_recent_changes(self, **kwargs):
        url = 'recents'
        return self._client._get(self._client.BASE_URL + url, **kwargs)



import os
from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(name='pipedrive-python-lib',
      version='1.1.3',
      description='API wrapper for Pipedrive written in Python',
      long_description=read('README.md'),
      long_description_content_type="text/markdown",
      url='https://github.com/GearPlug/pipedrive-python',
      author='Miguel Ferrer',
      author_email='ingferrermiguel@gmail.com',
      license='MIT',
      packages=['pipedrive'],
      install_requires=[
          'requests',
      ],
      zip_safe=False)



from urllib.parse import urlencode

import requests

from pipedrive import exceptions
from pipedrive.activities import Activities
from pipedrive.deals import Deals
from pipedrive.filters import Filters
from pipedrive.notes import Notes
from pipedrive.organizations import Organizations
from pipedrive.persons import Persons
from pipedrive.pipelines import Pipelines
from pipedrive.products import Products
from pipedrive.recents import Recents
from pipedrive.stages import Stages
from pipedrive.users import Users
from pipedrive.webhooks import Webhooks


class Client:
    BASE_URL = 'https://api-proxy.pipedrive.com/'
    OAUTH_BASE_URL = 'https://oauth.pipedrive.com/oauth/'

    def __init__(self, client_id=None, client_secret=None, domain=None):
        self.client_id = client_id
        self.client_secret = client_secret
        self.access_token = None
        self.api_token = None
        self.activities = Activities(self)
        self.deals = Deals(self)
        self.filters = Filters(self)
        self.notes = Notes(self)
        self.organizations = Organizations(self)
        self.persons = Persons(self)
        self.pipelines = Pipelines(self)
        self.products = Products(self)
        self.recents = Recents(self)
        self.stages = Stages(self)
        self.users = Users(self)
        self.webhooks = Webhooks(self)

        if domain:
            if not domain.endswith('/'):
                domain += '/'
            self.BASE_URL = domain + 'v1/'

    def authorization_url(self, redirect_uri, state=None):
        params = {
            'client_id': self.client_id,
            'redirect_uri': redirect_uri,
        }

        if state is not None:
            params['state'] = state

        return self.OAUTH_BASE_URL + 'authorize?' + urlencode(params)

    def exchange_code(self, redirect_uri, code):
        data = {
            'grant_type': 'authorization_code',
            'code': code,
            'redirect_uri': redirect_uri
        }
        return self._post(self.OAUTH_BASE_URL + 'token', data=data, auth=(self.client_id, self.client_secret))

    def refresh_token(self, refresh_token):
        data = {
            'grant_type': 'refresh_token',
            'refresh_token': refresh_token,
        }
        return self._post(self.OAUTH_BASE_URL + 'token', data=data, auth=(self.client_id, self.client_secret))

    def set_access_token(self, access_token):
        self.access_token = access_token

    def set_api_token(self, api_token):
        self.api_token = api_token

    def _get(self, url, **kwargs):
        return self._request('get', url, **kwargs)

    def _post(self, url, **kwargs):
        return self._request('post', url, **kwargs)

    def _put(self, url, **kwargs):
        return self._request('put', url, **kwargs)

    def _delete(self, url, **kwargs):
        return self._request('delete', url, **kwargs)

    def _request(self, method, url, headers=None, params=None, **kwargs):
        _headers = {}
        _params = {}
        if self.access_token:
            _headers['Authorization'] = 'Bearer {}'.format(self.access_token)
        if self.api_token:
            _params['api_token'] = self.api_token
        if headers:
            _headers.update(headers)
        if params:
            _params.update(params)
        return self._parse(requests.request(method, url, headers=_headers, params=_params, **kwargs))

    def _parse(self, response):
        status_code = response.status_code
        if 'Content-Type' in response.headers and 'application/json' in response.headers['Content-Type']:
            r = response.json()
        else:
            return response.text

        if not response.ok:
            error = None
            if 'error' in r:
                error = r['error']
            if status_code == 400:
                raise exceptions.BadRequestError(error, response)
            elif status_code == 401:
                raise exceptions.UnauthorizedError(error, response)
            elif status_code == 403:
                raise exceptions.ForbiddenError(error, response)
            elif status_code == 404:
                raise exceptions.NotFoundError(error, response)
            elif status_code == 410:
                raise exceptions.GoneError(error, response)
            elif status_code == 415:
                raise exceptions.UnsupportedMediaTypeError(error, response)
            elif status_code == 422:
                raise exceptions.UnprocessableEntityError(error, response)
            elif status_code == 429:
                raise exceptions.TooManyRequestsError(error, response)
            elif status_code == 500:
                raise exceptions.InternalServerError(error, response)
            elif status_code == 501:
                raise exceptions.NotImplementedError(error, response)
            elif status_code == 503:
                raise exceptions.ServiceUnavailableError(error, response)
            else:
                raise exceptions.UnknownError(error, response)

        return r



class Organizations(object):
    def __init__(self, client):
        self._client = client

    def get_organization(self, organization_id, **kwargs):
        url = 'organizations/{}'.format(organization_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_all_organizations(self, **kwargs):
        url = 'organizations'
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def create_organization(self, data, **kwargs):
        url = 'organizations'
        return self._client._post(self._client.BASE_URL + url, json=data, **kwargs)

    def update_organization(self, organization_id, data, **kwargs):
        url = 'organizations/{}'.format(organization_id)
        return self._client._put(self._client.BASE_URL + url, json=data, **kwargs)

    def delete_organization(self, organization_id, **kwargs):
        url = 'organizations/{}'.format(organization_id)
        return self._client._delete(self._client.BASE_URL + url, **kwargs)

    def get_organization_fields(self, **kwargs):
        url = 'organizationFields'
        return self._client._get(self._client.BASE_URL + url, **kwargs)
